<?php

namespace Project\Task\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Project\Task\Http\Requests\TasktypeRequest;
use Project\Task\Interfaces\TasktypeRepositoryInterface;
use Project\Task\Models\Tasktype;

/**
 * Resource controller class for tasktype.
 */
class TasktypeResourceController extends BaseController
{

    /**
     * Initialize tasktype resource controller.
     *
     * @param type TasktypeRepositoryInterface $tasktype
     *
     * @return null
     */
    public function __construct(TasktypeRepositoryInterface $tasktype)
    {
        parent::__construct();
        $this->repository = $tasktype;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Project\Task\Repositories\Criteria\TasktypeResourceCriteria::class);
    }

    /**
     * Display a list of tasktype.
     *
     * @return Response
     */
    public function index(TasktypeRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Project\Task\Repositories\Presenter\TasktypePresenter::class)
                ->$function();
        }

        $tasktypes = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('task::tasktype.names'))
            ->view('task::tasktype.index', true)
            ->data(compact('tasktypes', 'view'))
            ->output();
    }

    /**
     * Display tasktype.
     *
     * @param Request $request
     * @param Model   $tasktype
     *
     * @return Response
     */
    public function show(TasktypeRequest $request, Tasktype $tasktype)
    {

        if ($tasktype->exists) {
            $view = 'task::tasktype.show';
        } else {
            $view = 'task::tasktype.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('task::tasktype.name'))
            ->data(compact('tasktype'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new tasktype.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TasktypeRequest $request)
    {

        $tasktype = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('task::tasktype.name')) 
            ->view('task::tasktype.create', true) 
            ->data(compact('tasktype'))
            ->output();
    }

    /**
     * Create new tasktype.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TasktypeRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $tasktype                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('task::tasktype.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('task/tasktype/' . $tasktype->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/task/tasktype'))
                ->redirect();
        }

    }

    /**
     * Show tasktype for editing.
     *
     * @param Request $request
     * @param Model   $tasktype
     *
     * @return Response
     */
    public function edit(TasktypeRequest $request, Tasktype $tasktype)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('task::tasktype.name'))
            ->view('task::tasktype.edit', true)
            ->data(compact('tasktype'))
            ->output();
    }

    /**
     * Update the tasktype.
     *
     * @param Request $request
     * @param Model   $tasktype
     *
     * @return Response
     */
    public function update(TasktypeRequest $request, Tasktype $tasktype)
    {
        try {
            $attributes = $request->all();

            $tasktype->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('task::tasktype.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('task/tasktype/' . $tasktype->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('task/tasktype/' . $tasktype->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the tasktype.
     *
     * @param Model   $tasktype
     *
     * @return Response
     */
    public function destroy(TasktypeRequest $request, Tasktype $tasktype)
    {
        try {

            $tasktype->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('task::tasktype.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('task/tasktype/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('task/tasktype/' . $tasktype->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple tasktype.
     *
     * @param Model   $tasktype
     *
     * @return Response
     */
    public function delete(TasktypeRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('task::tasktype.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('task/tasktype'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/task/tasktype'))
                ->redirect();
        }

    }

    /**
     * Restore deleted tasktypes.
     *
     * @param Model   $tasktype
     *
     * @return Response
     */
    public function restore(TasktypeRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('task::tasktype.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/task/tasktype'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/task/tasktype/'))
                ->redirect();
        }

    }

}
